==> conc-2.0.1.tar.gz: conc-2.0.1/client/log.c <==
#include "log.h"
#include "error.h"
#include "client-message.h"
#include "log-query.h"
#include "date.h"

typedef struct _LogWindow {
    Conc *conc;
    gchar *log;
    GtkTextBuffer *buf;
    GtkTextView *tv;
    GladeXML *xml;
    MplexChannel chan;
    GIOChannel *io;
    guint reader;
    GtkTextIter iter;
    GtkTextMark *mark;
} LogWindow;

typedef struct _DateTimeDialog DateTimeDialog;

typedef struct _LogViewDialog {
    Conc *conc;
    GtkButton *start_but;
    guint64 start;
    DateTimeDialog *start_dialog;
    GtkButton *end_but;
    guint64 end;
    DateTimeDialog *end_dialog;
    GladeXML *xml;
} LogViewDialog;

struct _DateTimeDialog {
    LogViewDialog *lvd;
    gint start;
    GladeXML *xml;
};

/* 
 * Glade widget names:
 */
#define VIEW_LOG_DIALOG			"view log dialog"
#define VIEW_LOG_DIALOG_LOG		"log"
#define VIEW_LOG_DIALOG_START		"start"
#define VIEW_LOG_DIALOG_END		"end"
#define VIEW_LOG_DIALOG_LINES		"lines"
#define DATE_AND_TIME_DIALOG		"date and time dialog"
#define DATE_AND_TIME_DIALOG_DATE	"date"
#define DATE_AND_TIME_DIALOG_HOURS	"hours"
#define DATE_AND_TIME_DIALOG_MINUTES	"minutes"
#define DATE_AND_TIME_DIALOG_SECONDS	"seconds"
#define LOG_WINDOW			"log window"
#define LOG_WINDOW_TEXT			"textview"

/*
 * Glade handler names:
 */
#define VIEW_LOG_DIALOG_RESPONSE	"response"
#define VIEW_LOG_DIALOG_START_EDIT	"start date"
#define VIEW_LOG_DIALOG_END_EDIT	"end date"
#define DATE_AND_TIME_RESPONSE		"response"
#define LOG_WINDOW_RESPONSE		"response"

static void
log_window_closed(GtkDialog *dialog,
	gint arg1,
	gpointer user_data) {
    LogWindow *lw = user_data;

    gtk_widget_destroy(glade_xml_get_widget(lw->xml, LOG_WINDOW));
    lw->buf = NULL;
    /*FIXME: do something*/
}

static void
log_feeder_done(gpointer data) {
    LogWindow *lw = data;

    lw->reader = 0;
}

static gboolean
log_feeder(GIOChannel *source,
	GIOCondition condition,
	gpointer data) {
    LogWindow *lw = data;
    gchar buf[256];
    gsize bytes;
    gint i,start;
    GError *e = NULL;

    switch (g_io_channel_read_chars(source, buf, sizeof(buf), &bytes, &e)) {
	case G_IO_STATUS_ERROR:
	    /* FIXME: report error? */
	    g_clear_error(&e);
	    return FALSE;
	case G_IO_STATUS_NORMAL:
	    break;
	case G_IO_STATUS_EOF:
	    return FALSE;
	case G_IO_STATUS_AGAIN:
	    return TRUE;
	default:
	    g_assert_not_reached();
    }
    if (lw->buf == NULL) {
	/*FIXME: should do something else here? */
	return TRUE;
    }
    gtk_text_view_move_mark_onscreen(lw->tv, lw->mark);
    start = 0;
    for (i = 0; i < bytes ; i++) {
	if (buf[i] == '\n') {
	    gtk_text_buffer_insert(lw->buf, &lw->iter, &buf[start],
		    i - start + 1);
	    start = i + 1;
	    gtk_text_buffer_get_start_iter(lw->buf, &lw->iter);
	}
    }
    gtk_text_buffer_insert(lw->buf, &lw->iter, &buf[start],
	    i - start);
    gtk_text_view_scroll_mark_onscreen(lw->tv, lw->mark);
    return TRUE;
}

static void
log_opened(MessageQueue q,
	const MessageArg *arg,
	gpointer data) {
    GError *e = message_arg_error_check(arg);
    LogWindow *lw = data;

    if (e != NULL) {
	warning_dialog(lw->conc, _("Failed to open log"),
		_("Could not open log %s (%s)."),
		lw->log, e->message);
	g_clear_error(&e);
	g_free(lw->log);
	g_free(lw);
	return;
    }
    if (arg->type != MESSAGE_ARG_UINT16) {
	g_printerr(_("bad response from manager\n"));
	exit(EXIT_FAILURE);
    }
    lw->io = mplex_open(lw->conc->mplex, arg->uint16, NULL, &e);
    if (lw->io == NULL) {
	g_printerr(_("failed to open channel to log %s (%s)\n"),
		lw->log, e->message);
	exit(EXIT_FAILURE);
    }
    lw->xml = glade_xml_new(lw->conc->glade_file,
	    LOG_WINDOW, NULL);
    lw->tv = GTK_TEXT_VIEW(glade_xml_get_widget(lw->xml, LOG_WINDOW_TEXT));
    lw->buf = gtk_text_view_get_buffer(lw->tv);
    gtk_text_buffer_get_end_iter(lw->buf, &lw->iter);
    lw->mark = gtk_text_buffer_create_mark(lw->buf, NULL, &lw->iter, FALSE);
    gtk_text_view_scroll_mark_onscreen(lw->tv, lw->mark);
    gtk_text_buffer_get_start_iter(lw->buf, &lw->iter);
    glade_xml_signal_connect_data(lw->xml, LOG_WINDOW_RESPONSE,
	    (GCallback)log_window_closed, lw);
    lw->reader = g_io_add_watch_full(lw->io, G_PRIORITY_LOW, G_IO_IN,
	    log_feeder, lw, log_feeder_done);
}

static void
log_view_time_button(GtkButton *button,
	Date but_time) {
    gchar *timestr = date_print(but_time);

    gtk_button_set_label(button, timestr);
    g_free(timestr);
}

static void
log_view_update_buttons(LogViewDialog *lvd) {
    log_view_time_button(lvd->start_but, lvd->start);
    log_view_time_button(lvd->end_but, lvd->end);
}

static void
log_view_dtd_set(DateTimeDialog *dtd,
	Date dtd_time) {
    guint year = 0;
    guint month = 0;
    guint day = 0;
    guint hour = 0;
    guint minute = 0;
    guint second = 0;
    GtkCalendar *cal = GTK_CALENDAR(glade_xml_get_widget(dtd->xml,
		DATE_AND_TIME_DIALOG_DATE));

    date_explode(dtd_time, &year, &month, &day, &hour, &minute, &second);
    gtk_calendar_select_month(cal, month, year);
    gtk_calendar_select_day(cal, day);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(dtd->xml,
		    DATE_AND_TIME_DIALOG_HOURS)), (double)hour);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(dtd->xml,
		    DATE_AND_TIME_DIALOG_MINUTES)), (double)minute);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(dtd->xml,
		    DATE_AND_TIME_DIALOG_SECONDS)), (double)second);
}

static guint64
log_view_dtd_get(DateTimeDialog *dtd) {
    guint year = 0;
    guint month = 0;
    guint day = 0;
    guint hour = 0;
    guint minute = 0;
    guint second = 0;
    GtkCalendar *cal = GTK_CALENDAR(glade_xml_get_widget(dtd->xml,
		DATE_AND_TIME_DIALOG_DATE));

    gtk_calendar_get_date(cal, &year, &month, &day);
    hour = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(
		glade_xml_get_widget(dtd->xml, DATE_AND_TIME_DIALOG_HOURS)));
    minute = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(
		glade_xml_get_widget(dtd->xml, DATE_AND_TIME_DIALOG_MINUTES)));
    second = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(
		glade_xml_get_widget(dtd->xml, DATE_AND_TIME_DIALOG_SECONDS)));
    return date_new(year, month, day, hour, minute, second);
}

static void
log_view_dtd_close(DateTimeDialog *dtd) {
    gtk_widget_destroy(glade_xml_get_widget(dtd->xml, DATE_AND_TIME_DIALOG));
    if (dtd->start) {
	dtd->lvd->start_dialog = NULL;
    } else {
	dtd->lvd->end_dialog = NULL;
    }
    g_free(dtd);
}

static void
log_view_dtd_response(GtkDialog *dialog,
	gint arg1,
	gpointer user_data) {
    DateTimeDialog *dtd = user_data;

    if (arg1 == GTK_RESPONSE_OK) {
	guint64 val = log_view_dtd_get(dtd);

	if (dtd->start) {
	    if (val <= dtd->lvd->end) {
		dtd->lvd->start = val;
	    } else {
		dtd->lvd->start = dtd->lvd->end;
		dtd->lvd->end = val;
	    }
	} else {
	    if (val >= dtd->lvd->start) {
		dtd->lvd->end = val;
	    } else {
		dtd->lvd->end = dtd->lvd->start;
		dtd->lvd->start = val;
	    }
	}
	log_view_update_buttons(dtd->lvd);
    }
    log_view_dtd_close(dtd);
}

static void
log_view_edit_date(GtkButton *button,
	gpointer user_data) {
    LogViewDialog *lvd = user_data;
    gint start = (button == lvd->start_but);
    DateTimeDialog *dtd = NULL;

    if (start) {
	dtd = lvd->start_dialog;
    } else {
	dtd = lvd->end_dialog;
    }
    if (dtd != NULL) {
	gtk_window_present(GTK_WINDOW(glade_xml_get_widget(dtd->xml,
			DATE_AND_TIME_DIALOG)));
    } else {
	dtd = g_new0(struct _DateTimeDialog, 1);
	dtd->lvd = lvd;
	dtd->start = start;
	dtd->xml = glade_xml_new(lvd->conc->glade_file,
		DATE_AND_TIME_DIALOG, NULL);
	if (start) {
	    lvd->start_dialog = dtd;
	    log_view_dtd_set(dtd, lvd->start);
	} else {
	    lvd->end_dialog = dtd;
	    log_view_dtd_set(dtd, lvd->end);
	}
	glade_xml_signal_connect_data(dtd->xml, DATE_AND_TIME_RESPONSE,
		(GCallback)log_view_dtd_response, dtd);
    }
}

static void
log_view_response(GtkDialog *dialog,
	gint arg1,
	gpointer user_data) {
    LogViewDialog *lvd = user_data;
    GError *e = NULL;
    MessageArg *a;

    if (arg1 == GTK_RESPONSE_OK) {
	LogWindow *lw = g_new0(struct _LogWindow, 1);
	LogQuery *lq;

	lw->conc = lvd->conc;
	lw->log = g_strdup(gtk_entry_get_text(GTK_ENTRY(GTK_BIN(
			    glade_xml_get_widget(lvd->xml,
			    VIEW_LOG_DIALOG_LOG))->child)));
	lq = log_query_new(lw->log, lvd->start, lvd->end,
		atoi(gtk_entry_get_text(GTK_ENTRY(
				glade_xml_get_widget(lvd->xml,
				    VIEW_LOG_DIALOG_LINES)))), NULL);
	a = log_query_message_arg("query", lq);
	if (message_send(lvd->conc->msg_q, CLIENT_MESSAGE_LOG, a,
		    log_opened, lw, &e) == FALSE) {
	    g_printerr("failed to send message (%s)\n", e->message);
	    g_free(lw->log);
	    g_free(lw);
	}
	log_query_free(lq);
	message_arg_free(a);
    }
    gtk_widget_destroy(GTK_WIDGET(dialog));
    if (lvd->start_dialog) {
	log_view_dtd_close(lvd->start_dialog);
    }
    if (lvd->end_dialog) {
	log_view_dtd_close(lvd->end_dialog);
    }
    g_free(lvd);
}

static void
console_logs(MessageQueue q,
	const MessageArg *arg,
	gpointer data) {
    GError *e = message_arg_error_check(arg);
    LogViewDialog *lvd = data;
    GtkListStore *model;
    GtkTreeIter iter;
    GList *l;

    if (e != NULL) {
	warning_dialog(lvd->conc, _("Could not fetch list of logs"),
		_("Could not fetch list of logs for console (%s)."),
		e->message);
	g_clear_error(&e);
	return;
    }
    if (arg->type != MESSAGE_ARG_LIST) {
	g_printerr(_("bad response from manager\n"));
	exit(EXIT_FAILURE);
    }
    if (arg->list == NULL) {
	return;
    }
    model = gtk_list_store_new(LOGS_COLUMN_LAST, G_TYPE_STRING);
    gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(model),
	    LOGS_COLUMN_NAME, GTK_SORT_ASCENDING);
    for (l = arg->list; l != NULL; l = g_list_next(l)) {
	MessageArg *a = l->data;

	if (a->type != MESSAGE_ARG_STRING) {
	    continue;
	}
	gtk_list_store_append(model, &iter);
	gtk_list_store_set(model, &iter,
		LOGS_COLUMN_NAME, a->string->str,
		-1);
    }
    gtk_combo_box_set_model(GTK_COMBO_BOX(glade_xml_get_widget(lvd->xml,
		    VIEW_LOG_DIALOG_LOG)), GTK_TREE_MODEL(model));
    if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(model),
		&iter) == TRUE) {
	gchar *log = NULL;

	gtk_tree_model_get(GTK_TREE_MODEL(model), &iter,
		LOGS_COLUMN_NAME, &log, -1);
	if (log != NULL) {
	    gtk_entry_set_text(GTK_ENTRY(GTK_BIN(
			    glade_xml_get_widget(lvd->xml,
			    VIEW_LOG_DIALOG_LOG))->child), log);
	    g_free(log);
	}
    }
    g_object_unref(G_OBJECT(model)); /*model gets destroyed with combobox*/
}


extern void
log_view(Conc *conc,
	const char *console) {
    LogViewDialog *lvd = g_new0(LogViewDialog, 1);
    Date now = date_now();
    GtkTreeIter iter;

    lvd->conc = conc;
    lvd->start = now - (24 * 60 * 60);
    lvd->end = now;
    lvd->xml = glade_xml_new(conc->glade_file, VIEW_LOG_DIALOG, NULL);
    lvd->start_but = GTK_BUTTON(glade_xml_get_widget(lvd->xml,
		VIEW_LOG_DIALOG_START));
    lvd->end_but = GTK_BUTTON(glade_xml_get_widget(lvd->xml,
		VIEW_LOG_DIALOG_END));
    log_view_update_buttons(lvd);
    gtk_combo_box_set_model(GTK_COMBO_BOX(glade_xml_get_widget(lvd->xml,
		    VIEW_LOG_DIALOG_LOG)), GTK_TREE_MODEL(conc->all_logs));
    gtk_combo_box_entry_set_text_column(GTK_COMBO_BOX_ENTRY(
		glade_xml_get_widget(lvd->xml, VIEW_LOG_DIALOG_LOG)),
	    LOGS_COLUMN_NAME);
    if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(conc->all_logs),
		&iter) == TRUE) {
	gchar *log = NULL;

	gtk_tree_model_get(GTK_TREE_MODEL(conc->all_logs), &iter,
		LOGS_COLUMN_NAME, &log, -1);
	if (log != NULL) {
	    gtk_entry_set_text(GTK_ENTRY(GTK_BIN(
			    glade_xml_get_widget(lvd->xml,
			    VIEW_LOG_DIALOG_LOG))->child), log);
	    g_free(log);
	}
    }
    glade_xml_signal_connect_data(lvd->xml, VIEW_LOG_DIALOG_RESPONSE,
	    (GCallback)log_view_response, lvd);
    glade_xml_signal_connect_data(lvd->xml, VIEW_LOG_DIALOG_START_EDIT,
	    (GCallback)log_view_edit_date, lvd);
    glade_xml_signal_connect_data(lvd->xml, VIEW_LOG_DIALOG_END_EDIT,
	    (GCallback)log_view_edit_date, lvd);
    if (console != NULL) {
	MessageArg *a;
	GError *e = NULL;

	a = message_arg_string("console", console);
	if (message_send(lvd->conc->msg_q, CLIENT_MESSAGE_CONSOLE_LOGS, a,
		    console_logs, lvd, &e) == FALSE) {
	    g_printerr("failed to send message (%s)\n", e->message);
	}
	message_arg_free(a);
    }
}

==> conc-2.0.3.tar.gz: conc-2.0.3/client/log.c <==
#include "log.h"
#include "error.h"
#include "client-message.h"
#include "log-query.h"
#include "date.h"

typedef struct _LogWindow {
    Conc *conc;
    gchar *log;
    GtkTextBuffer *buf;
    GtkTextView *tv;
    GladeXML *xml;
    MplexChannel chan;
    GIOChannel *io;
    guint reader;
    GtkTextIter iter;
    GtkTextMark *mark;
} LogWindow;

typedef struct _DateTimeDialog DateTimeDialog;

typedef struct _LogViewDialog {
    Conc *conc;
    GtkButton *start_but;
    guint64 start;
    DateTimeDialog *start_dialog;
    GtkButton *end_but;
    guint64 end;
    DateTimeDialog *end_dialog;
    GladeXML *xml;
} LogViewDialog;

struct _DateTimeDialog {
    LogViewDialog *lvd;
    gint start;
    GladeXML *xml;
};

/* 
 * Glade widget names:
 */
#define VIEW_LOG_DIALOG			"view log dialog"
#define VIEW_LOG_DIALOG_LOG		"vldlog"
#define VIEW_LOG_DIALOG_START		"vldstart"
#define VIEW_LOG_DIALOG_END		"vldend"
#define VIEW_LOG_DIALOG_LINES		"vldlines"
#define DATE_AND_TIME_DIALOG		"date and time dialog"
#define DATE_AND_TIME_DIALOG_DATE	"datdate"
#define DATE_AND_TIME_DIALOG_HOURS	"dathours"
#define DATE_AND_TIME_DIALOG_MINUTES	"datminutes"
#define DATE_AND_TIME_DIALOG_SECONDS	"datseconds"
#define LOG_WINDOW			"log window"
#define LOG_WINDOW_TEXT			"lwtextview"

/*
 * Glade handler names:
 */
#define VIEW_LOG_DIALOG_RESPONSE	"response"
#define VIEW_LOG_DIALOG_START_EDIT	"start date"
#define VIEW_LOG_DIALOG_END_EDIT	"end date"
#define DATE_AND_TIME_RESPONSE		"response"
#define LOG_WINDOW_RESPONSE		"response"

static void
log_window_closed(GtkDialog *dialog,
	gint arg1,
	gpointer user_data) {
    LogWindow *lw = user_data;

    gtk_widget_destroy(glade_xml_get_widget(lw->xml, LOG_WINDOW));
    lw->buf = NULL;
    /*FIXME: do something*/
}

static void
log_feeder_done(gpointer data) {
    LogWindow *lw = data;

    lw->reader = 0;
}

static gboolean
log_feeder(GIOChannel *source,
	GIOCondition condition,
	gpointer data) {
    LogWindow *lw = data;
    gchar buf[256];
    gsize bytes;
    gint i,start;
    GError *e = NULL;

    switch (g_io_channel_read_chars(source, buf, sizeof(buf), &bytes, &e)) {
	case G_IO_STATUS_ERROR:
	    /* FIXME: report error? */
	    g_clear_error(&e);
	    return FALSE;
	case G_IO_STATUS_NORMAL:
	    break;
	case G_IO_STATUS_EOF:
	    return FALSE;
	case G_IO_STATUS_AGAIN:
	    return TRUE;
	default:
	    g_assert_not_reached();
    }
    if (lw->buf == NULL) {
	/*FIXME: should do something else here? */
	return TRUE;
    }
    gtk_text_view_move_mark_onscreen(lw->tv, lw->mark);
    start = 0;
    for (i = 0; i < bytes ; i++) {
	if (buf[i] == '\n') {
	    gtk_text_buffer_insert(lw->buf, &lw->iter, &buf[start],
		    i - start + 1);
	    start = i + 1;
	    gtk_text_buffer_get_start_iter(lw->buf, &lw->iter);
	}
    }
    gtk_text_buffer_insert(lw->buf, &lw->iter, &buf[start],
	    i - start);
    gtk_text_view_scroll_mark_onscreen(lw->tv, lw->mark);
    return TRUE;
}

static void
log_opened(MessageQueue q,
	const MessageArg *arg,
	gpointer data) {
    GError *e = message_arg_error_check(arg);
    LogWindow *lw = data;

    if (e != NULL) {
	warning_dialog(lw->conc, _("Failed to open log"),
		_("Could not open log %s (%s)."),
		lw->log, e->message);
	g_clear_error(&e);
	g_free(lw->log);
	g_free(lw);
	return;
    }
    if (arg->type != MESSAGE_ARG_UINT16) {
	g_printerr(_("bad response from manager\n"));
	exit(EXIT_FAILURE);
    }
    lw->io = mplex_open(lw->conc->mplex, arg->uint16, NULL, NULL, NULL, &e);
    if (lw->io == NULL) {
	g_printerr(_("failed to open channel to log %s (%s)\n"),
		lw->log, e->message);
	exit(EXIT_FAILURE);
    }
    lw->xml = glade_xml_new(lw->conc->glade_file,
	    LOG_WINDOW, NULL);
    lw->tv = GTK_TEXT_VIEW(glade_xml_get_widget(lw->xml, LOG_WINDOW_TEXT));
    lw->buf = gtk_text_view_get_buffer(lw->tv);
    gtk_text_buffer_get_end_iter(lw->buf, &lw->iter);
    lw->mark = gtk_text_buffer_create_mark(lw->buf, NULL, &lw->iter, FALSE);
    gtk_text_view_scroll_mark_onscreen(lw->tv, lw->mark);
    gtk_text_buffer_get_start_iter(lw->buf, &lw->iter);
    glade_xml_signal_connect_data(lw->xml, LOG_WINDOW_RESPONSE,
	    (GCallback)log_window_closed, lw);
    lw->reader = g_io_add_watch_full(lw->io, G_PRIORITY_LOW, G_IO_IN,
	    log_feeder, lw, log_feeder_done);
}

static void
log_view_time_button(GtkButton *button,
	Date but_time) {
    gchar *timestr = date_print(but_time);

    gtk_button_set_label(button, timestr);
    g_free(timestr);
}

static void
log_view_update_buttons(LogViewDialog *lvd) {
    log_view_time_button(lvd->start_but, lvd->start);
    log_view_time_button(lvd->end_but, lvd->end);
}

static void
log_view_dtd_set(DateTimeDialog *dtd,
	Date dtd_time) {
    guint year = 0;
    guint month = 0;
    guint day = 0;
    guint hour = 0;
    guint minute = 0;
    guint second = 0;
    GtkCalendar *cal = GTK_CALENDAR(glade_xml_get_widget(dtd->xml,
		DATE_AND_TIME_DIALOG_DATE));

    date_explode(dtd_time, &year, &month, &day, &hour, &minute, &second);
    gtk_calendar_select_month(cal, month, year);
    gtk_calendar_select_day(cal, day);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(dtd->xml,
		    DATE_AND_TIME_DIALOG_HOURS)), (double)hour);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(dtd->xml,
		    DATE_AND_TIME_DIALOG_MINUTES)), (double)minute);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(dtd->xml,
		    DATE_AND_TIME_DIALOG_SECONDS)), (double)second);
}

static guint64
log_view_dtd_get(DateTimeDialog *dtd) {
    guint year = 0;
    guint month = 0;
    guint day = 0;
    guint hour = 0;
    guint minute = 0;
    guint second = 0;
    GtkCalendar *cal = GTK_CALENDAR(glade_xml_get_widget(dtd->xml,
		DATE_AND_TIME_DIALOG_DATE));

    gtk_calendar_get_date(cal, &year, &month, &day);
    hour = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(
		glade_xml_get_widget(dtd->xml, DATE_AND_TIME_DIALOG_HOURS)));
    minute = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(
		glade_xml_get_widget(dtd->xml, DATE_AND_TIME_DIALOG_MINUTES)));
    second = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(
		glade_xml_get_widget(dtd->xml, DATE_AND_TIME_DIALOG_SECONDS)));
    return date_new(year, month, day, hour, minute, second);
}

static void
log_view_dtd_close(DateTimeDialog *dtd) {
    gtk_widget_destroy(glade_xml_get_widget(dtd->xml, DATE_AND_TIME_DIALOG));
    if (dtd->start) {
	dtd->lvd->start_dialog = NULL;
    } else {
	dtd->lvd->end_dialog = NULL;
    }
    g_free(dtd);
}

static void
log_view_dtd_response(GtkDialog *dialog,
	gint arg1,
	gpointer user_data) {
    DateTimeDialog *dtd = user_data;

    if (arg1 == GTK_RESPONSE_OK) {
	guint64 val = log_view_dtd_get(dtd);

	if (dtd->start) {
	    if (val <= dtd->lvd->end) {
		dtd->lvd->start = val;
	    } else {
		dtd->lvd->start = dtd->lvd->end;
		dtd->lvd->end = val;
	    }
	} else {
	    if (val >= dtd->lvd->start) {
		dtd->lvd->end = val;
	    } else {
		dtd->lvd->end = dtd->lvd->start;
		dtd->lvd->start = val;
	    }
	}
	log_view_update_buttons(dtd->lvd);
    }
    log_view_dtd_close(dtd);
}

static void
log_view_edit_date(GtkButton *button,
	gpointer user_data) {
    LogViewDialog *lvd = user_data;
    gint start = (button == lvd->start_but);
    DateTimeDialog *dtd = NULL;

    if (start) {
	dtd = lvd->start_dialog;
    } else {
	dtd = lvd->end_dialog;
    }
    if (dtd != NULL) {
	gtk_window_present(GTK_WINDOW(glade_xml_get_widget(dtd->xml,
			DATE_AND_TIME_DIALOG)));
    } else {
	dtd = g_new0(struct _DateTimeDialog, 1);
	dtd->lvd = lvd;
	dtd->start = start;
	dtd->xml = glade_xml_new(lvd->conc->glade_file,
		DATE_AND_TIME_DIALOG, NULL);
	if (start) {
	    lvd->start_dialog = dtd;
	    log_view_dtd_set(dtd, lvd->start);
	} else {
	    lvd->end_dialog = dtd;
	    log_view_dtd_set(dtd, lvd->end);
	}
	glade_xml_signal_connect_data(dtd->xml, DATE_AND_TIME_RESPONSE,
		(GCallback)log_view_dtd_response, dtd);
    }
}

static void
log_view_response(GtkDialog *dialog,
	gint arg1,
	gpointer user_data) {
    LogViewDialog *lvd = user_data;
    GError *e = NULL;
    MessageArg *a;

    if (arg1 == GTK_RESPONSE_OK) {
	LogWindow *lw = g_new0(struct _LogWindow, 1);
	LogQuery *lq;

	lw->conc = lvd->conc;
	lw->log = g_strdup(gtk_entry_get_text(GTK_ENTRY(GTK_BIN(
			    glade_xml_get_widget(lvd->xml,
			    VIEW_LOG_DIALOG_LOG))->child)));
	lq = log_query_new(lw->log, lvd->start, lvd->end,
		atoi(gtk_entry_get_text(GTK_ENTRY(
				glade_xml_get_widget(lvd->xml,
				    VIEW_LOG_DIALOG_LINES)))), NULL);
	a = log_query_message_arg("query", lq);
	if (message_send(lvd->conc->msg_q, CLIENT_MESSAGE_LOG, a,
		    log_opened, lw, &e) == FALSE) {
	    g_printerr("failed to send message (%s)\n", e->message);
	    g_free(lw->log);
	    g_free(lw);
	}
	log_query_free(lq);
	message_arg_free(a);
    }
    gtk_widget_destroy(GTK_WIDGET(dialog));
    if (lvd->start_dialog) {
	log_view_dtd_close(lvd->start_dialog);
    }
    if (lvd->end_dialog) {
	log_view_dtd_close(lvd->end_dialog);
    }
    g_free(lvd);
}

static void
console_logs(MessageQueue q,
	const MessageArg *arg,
	gpointer data) {
    GError *e = message_arg_error_check(arg);
    LogViewDialog *lvd = data;
    GtkListStore *model;
    GtkTreeIter iter;
    GList *l;

    if (e != NULL) {
	warning_dialog(lvd->conc, _("Could not fetch list of logs"),
		_("Could not fetch list of logs for console (%s)."),
		e->message);
	g_clear_error(&e);
	return;
    }
    if (arg->type != MESSAGE_ARG_LIST) {
	g_printerr(_("bad response from manager\n"));
	exit(EXIT_FAILURE);
    }
    if (arg->list == NULL) {
	return;
    }
    model = gtk_list_store_new(LOGS_COLUMN_LAST, G_TYPE_STRING);
    gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(model),
	    LOGS_COLUMN_NAME, GTK_SORT_ASCENDING);
    for (l = arg->list; l != NULL; l = g_list_next(l)) {
	MessageArg *a = l->data;

	if (a->type != MESSAGE_ARG_STRING) {
	    continue;
	}
	gtk_list_store_append(model, &iter);
	gtk_list_store_set(model, &iter,
		LOGS_COLUMN_NAME, a->string->str,
		-1);
    }
    gtk_combo_box_set_model(GTK_COMBO_BOX(glade_xml_get_widget(lvd->xml,
		    VIEW_LOG_DIALOG_LOG)), GTK_TREE_MODEL(model));
    if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(model),
		&iter) == TRUE) {
	gchar *log = NULL;

	gtk_tree_model_get(GTK_TREE_MODEL(model), &iter,
		LOGS_COLUMN_NAME, &log, -1);
	if (log != NULL) {
	    gtk_entry_set_text(GTK_ENTRY(GTK_BIN(
			    glade_xml_get_widget(lvd->xml,
			    VIEW_LOG_DIALOG_LOG))->child), log);
	    g_free(log);
	}
    }
    g_object_unref(G_OBJECT(model)); /*model gets destroyed with combobox*/
}


extern void
log_view(Conc *conc,
	const char *console) {
    LogViewDialog *lvd = g_new0(LogViewDialog, 1);
    Date now = date_now();
    GtkTreeIter iter;

    lvd->conc = conc;
    lvd->start = now - (24 * 60 * 60);
    lvd->end = now;
    lvd->xml = glade_xml_new(conc->glade_file, VIEW_LOG_DIALOG, NULL);
    lvd->start_but = GTK_BUTTON(glade_xml_get_widget(lvd->xml,
		VIEW_LOG_DIALOG_START));
    lvd->end_but = GTK_BUTTON(glade_xml_get_widget(lvd->xml,
		VIEW_LOG_DIALOG_END));
    log_view_update_buttons(lvd);
    gtk_combo_box_set_model(GTK_COMBO_BOX(glade_xml_get_widget(lvd->xml,
		    VIEW_LOG_DIALOG_LOG)), GTK_TREE_MODEL(conc->all_logs));
    gtk_combo_box_entry_set_text_column(GTK_COMBO_BOX_ENTRY(
		glade_xml_get_widget(lvd->xml, VIEW_LOG_DIALOG_LOG)),
	    LOGS_COLUMN_NAME);
    if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(conc->all_logs),
		&iter) == TRUE) {
	gchar *log = NULL;

	gtk_tree_model_get(GTK_TREE_MODEL(conc->all_logs), &iter,
		LOGS_COLUMN_NAME, &log, -1);
	if (log != NULL) {
	    gtk_entry_set_text(GTK_ENTRY(GTK_BIN(
			    glade_xml_get_widget(lvd->xml,
			    VIEW_LOG_DIALOG_LOG))->child), log);
	    g_free(log);
	}
    }
    glade_xml_signal_connect_data(lvd->xml, VIEW_LOG_DIALOG_RESPONSE,
	    (GCallback)log_view_response, lvd);
    glade_xml_signal_connect_data(lvd->xml, VIEW_LOG_DIALOG_START_EDIT,
	    (GCallback)log_view_edit_date, lvd);
    glade_xml_signal_connect_data(lvd->xml, VIEW_LOG_DIALOG_END_EDIT,
	    (GCallback)log_view_edit_date, lvd);
    if (console != NULL) {
	MessageArg *a;
	GError *e = NULL;

	a = message_arg_string("console", console);
	if (message_send(lvd->conc->msg_q, CLIENT_MESSAGE_CONSOLE_LOGS, a,
		    console_logs, lvd, &e) == FALSE) {
	    g_printerr("failed to send message (%s)\n", e->message);
	}
	message_arg_free(a);
    }
}
